#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from __future__ import unicode_literals
import sys
from django.conf import settings

# Detect if django.db.migrations is supported
from django import VERSION as DJANGO_VERSION
NATIVE_MIGRATIONS = (DJANGO_VERSION >= (1, 7))


INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'djmoney',
    'djmoney.tests.testapp',
    'reversion',
)

if not NATIVE_MIGRATIONS:
    INSTALLED_APPS += (
        'south',
    )

settings.configure(
    DEBUG=True,
    DATABASES={
         'default': {
             'ENGINE': 'django.db.backends.sqlite3',
         }
    },
    SITE_ID=1,
    ROOT_URLCONF=None,
    MIDDLEWARE_CLASSES=(
        'django.middleware.common.CommonMiddleware',
    ),
    INSTALLED_APPS=INSTALLED_APPS,
    USE_TZ=True,
    USE_L10N=True,
    SOUTH_TESTS_MIGRATE=True,
)

import moneyed
from moneyed.localization import _FORMATTER, DEFAULT
from decimal import ROUND_HALF_EVEN

_FORMATTER.add_sign_definition('pl_PL', moneyed.PLN, suffix=' zł')
_FORMATTER.add_sign_definition(DEFAULT, moneyed.PLN, suffix=' zł')
_FORMATTER.add_formatting_definition(
     "pl_PL", group_size=3, group_separator=" ", decimal_point=",",
     positive_sign="", trailing_positive_sign="",
     negative_sign="-", trailing_negative_sign="",
     rounding_method=ROUND_HALF_EVEN)


try:
    from django.test.simple import DjangoTestSuiteRunner
except ImportError:
    from django.test.runner import DiscoverRunner as DjangoTestSuiteRunner

test_runner = DjangoTestSuiteRunner(verbosity=1, failfast=False)

# Native migrations are present in Django 1.7+
# This also requires initializing the app registry with django.setup()
# If native migrations are not present, initialize South and configure it for running the test suite
if NATIVE_MIGRATIONS:
    from django import setup
    setup()
else:
    from south.management.commands import patch_for_test_db_setup
    patch_for_test_db_setup()

if len(sys.argv) > 1:
    tests = sys.argv[1:]
else:
    tests = ['djmoney']
failures = test_runner.run_tests(tests)
if failures:
    sys.exit(failures)


## Run py.tests
# Compatibility testing patches on the py-moneyed
import pytest
failures = pytest.main()

if failures:
    sys.exit(failures)



from __future__ import unicode_literals
try:
    from django.utils.encoding import smart_unicode
except ImportError:
    # Python 3
    from django.utils.encoding import smart_text as smart_unicode

try:
    from django.utils.timezone import localtime
except ImportError:
    def localtime(value):
        return value

from django.core.exceptions import ObjectDoesNotExist
try:
    from django.contrib.admin.utils import lookup_field
except ImportError:
    from django.contrib.admin.util import lookup_field
from django.utils.safestring import mark_safe
from django.utils.html import conditional_escape
from django.db.models.fields.related import ManyToManyRel


def get_empty_value_display(cls):
    if hasattr(cls.model_admin, 'get_empty_value_display'):
        return cls.model_admin.get_empty_value_display()
    else:
        # Django < 1.9
        from django.contrib.admin.views.main import EMPTY_CHANGELIST_VALUE
        return EMPTY_CHANGELIST_VALUE


def djmoney_contents(self):
    from django.contrib.admin.templatetags.admin_list import _boolean_icon

    field, obj, model_admin = self.field['field'], self.form.instance, self.model_admin

    try:
        f, attr, value = lookup_field(field, obj, model_admin)
    except (AttributeError, ValueError, ObjectDoesNotExist):
        result_repr = get_empty_value_display(self)
    else:
        if f is None:
            boolean = getattr(attr, "boolean", False)
            if boolean:
                result_repr = _boolean_icon(value)
            else:
                result_repr = smart_unicode(value)
                if getattr(attr, "allow_tags", False):
                    result_repr = mark_safe(result_repr)
        else:
            if value is None:
                result_repr = get_empty_value_display(self)
            elif isinstance(f.rel, ManyToManyRel):
                result_repr = ", ".join(map(str, value.all()))
            else:
                result_repr = smart_unicode(value)
    return conditional_escape(result_repr)


from django.contrib.admin.helpers import AdminReadonlyField

AdminReadonlyField.contents = djmoney_contents



import operator

from django.conf import settings
from moneyed import CURRENCIES, DEFAULT_CURRENCY_CODE, DEFAULT_CURRENCY

# The default currency, you can define this in your project's settings module
# This has to be a currency object imported from moneyed
DEFAULT_CURRENCY = getattr(settings, 'DEFAULT_CURRENCY', DEFAULT_CURRENCY)


# The default currency choices, you can define this in your project's
# settings module
PROJECT_CURRENCIES = getattr(settings, 'CURRENCIES', None)

if PROJECT_CURRENCIES:
    CURRENCY_CHOICES = [(code, CURRENCIES[code].name) for code in
                        PROJECT_CURRENCIES]
else:
    CURRENCY_CHOICES = [(c.code, c.name) for i, c in CURRENCIES.items() if
                        c.code != DEFAULT_CURRENCY_CODE]

CURRENCY_CHOICES.sort(key=operator.itemgetter(1, 0))



from functools import wraps

import django
try:
    from django.db.models.expressions import BaseExpression, F
except ImportError:
    # Django < 1.8
    from django.db.models.expressions import ExpressionNode as BaseExpression, F
from django.db.models.sql.query import Query
from djmoney.models.fields import MoneyField
from django.db.models.query_utils import Q
from moneyed import Money


try:
    from django.utils.encoding import smart_unicode
except ImportError:
    # Python 3
    from django.utils.encoding import smart_text as smart_unicode

from djmoney.utils import get_currency_field_name

try:
    from django.db.models.constants import LOOKUP_SEP
except ImportError:
    # Django < 1.5
    LOOKUP_SEP = '__'

from django.db.models.sql.constants import QUERY_TERMS


def _get_clean_name(name):

    # Get rid of __lt, __gt etc for the currency lookup
    path = name.split(LOOKUP_SEP)
    if path[-1] in QUERY_TERMS:
        return LOOKUP_SEP.join(path[:-1])
    else:
        return name


def _get_field(model, name):
    from django.db.models.fields import FieldDoesNotExist

    # Create a fake query object so we can easily work out what field
    # type we are dealing with
    qs = Query(model)
    opts = qs.get_meta()
    alias = qs.get_initial_alias()

    parts = name.split(LOOKUP_SEP)

    # The following is borrowed from the innards of Query.add_filter - it strips out __gt, __exact et al.
    num_parts = len(parts)
    if num_parts > 1 and parts[-1] in qs.query_terms:
        # Traverse the lookup query to distinguish related fields from
        # lookup types.
        lookup_model = model
        for counter, field_name in enumerate(parts):
            try:
                lookup_field = lookup_model._meta.get_field(field_name)
            except FieldDoesNotExist:
                # Not a field. Bail out.
                parts.pop()
                break
            # Unless we're at the end of the list of lookups, let's attempt
            # to continue traversing relations.
            if (counter + 1) < num_parts:
                try:
                    lookup_model = lookup_field.rel.to
                except AttributeError:
                    # Not a related field. Bail out.
                    parts.pop()
                    break

    if django.VERSION[0] >= 1 and django.VERSION[1] in (6, 7):
        # Django 1.6-1.7
        field = qs.setup_joins(parts, opts, alias)[0]
    else:
        # Django 1.4-1.5
        field = qs.setup_joins(parts, opts, alias, False)[0]

    return field


def _expand_money_args(model, args):
    """
    Augments args so that they contain _currency lookups - ie.. Q() | Q()
    """
    for arg in args:
        if isinstance(arg, Q):
            for i, child in enumerate(arg.children):
                if isinstance(child, Q):
                    _expand_money_args(model, [child])
                elif isinstance(child, (list, tuple)):
                    name, value = child
                    if isinstance(value, Money):
                        clean_name = _get_clean_name(name)
                        arg.children[i] = Q(*[
                            child,
                            (get_currency_field_name(clean_name), smart_unicode(value.currency))
                        ])
                    if isinstance(value, BaseExpression):
                        field = _get_field(model, name)
                        if isinstance(field, MoneyField):
                            clean_name = _get_clean_name(name)
                            arg.children[i] = Q(*[
                                child, 
                                ('_'.join([clean_name, 'currency']), F(get_currency_field_name(value.name)))
                            ])
    return args


def _expand_money_kwargs(model, kwargs):
    """
    Augments kwargs so that they contain _currency lookups.
    """
    to_append = {}
    for name, value in kwargs.items():
        if isinstance(value, Money):
            clean_name = _get_clean_name(name)
            to_append[name] = value.amount
            to_append[get_currency_field_name(clean_name)] = smart_unicode(
                value.currency)
        cmp_cls = BaseExpression
        if django.VERSION >= (1, 8):
            cmp_cls = F
        if isinstance(value, cmp_cls):
            field = _get_field(model, name)
            if isinstance(field, MoneyField):
                clean_name = _get_clean_name(name)
                to_append['_'.join([clean_name, 'currency'])] = F(get_currency_field_name(value.name))

    kwargs.update(to_append)
    return kwargs


def understands_money(model, func):
    """
    Used to wrap a queryset method with logic to expand
    a query from something like:

    mymodel.objects.filter(money=Money(100,"USD"))

    To something equivalent to:

    mymodel.objects.filter(money=Decimal("100.0), money_currency="USD")
    """

    @wraps(func)
    def wrapper(*args, **kwargs):
        args = _expand_money_args(model, args)
        kwargs = kwargs.copy()
        kwargs = _expand_money_kwargs(model, kwargs)
        return func(*args, **kwargs)

    return wrapper


RELEVANT_QUERYSET_METHODS = ['distinct', 'get', 'get_or_create', 'filter',
                             'exclude']


def add_money_comprehension_to_queryset(model, qs):
    # Decorate each relevant method with understand_money in the queryset given
    for attr in RELEVANT_QUERYSET_METHODS:
        setattr(qs, attr, understands_money(model, getattr(qs, attr)))
    return qs


def money_manager(manager):
    """
    Patches a model manager's get_queryset method so that each QuerySet it returns
    is able to work on money fields.

    This allow users of django-money to use other managers while still doing
    money queries.
    """

    # Need to dynamically subclass to add our behaviour, and then change
    # the class of 'manager' to our subclass.

    # Rejected alternatives:
    #
    # * A monkey patch that adds things to the manager instance dictionary.
    #   This fails due to complications with Manager._copy_to_model behaviour.
    #
    # * Returning a new MoneyManager instance (rather than modifying
    #   the passed in manager instance). This fails for reasons that
    #   are tricky to get to the bottom of - Manager does funny things.
    class MoneyManager(manager.__class__):

        def get_queryset(self, *args, **kwargs):
            # If we are calling code that is pre-Django 1.6, need to
            # spell it 'get_query_set'
            s = super(MoneyManager, self)
            method = getattr(s, 'get_queryset',
                             getattr(s, 'get_query_set', None))
            return add_money_comprehension_to_queryset(self.model, method(*args, **kwargs))

        # If we are being called by code pre Django 1.6, need
        # 'get_query_set'.
        if django.VERSION < (1, 6):
            get_query_set = get_queryset

    manager.__class__ = MoneyManager
    return manager



# -*- encoding: utf-8
from moneyed import test_moneyed_classes
from djmoney.models.fields import MoneyPatched


class TestDjangoMoney(test_moneyed_classes.TestMoney):

    # MoneyPatched localizes to 'en_US', removing "US" from the default
    # py-moneyed prefix, 'US$'. This breaks py-moneyed's test_str.
    def test_str(self):
        assert str(self.one_million_bucks) == '$1,000,000.00'

# replace class "Money" a class "MoneyPath"
test_moneyed_classes.Money = MoneyPatched

TestCurrency = test_moneyed_classes.TestCurrency

TestMoney = TestDjangoMoney



from __future__ import division
from django import VERSION
from django.db import models
from django.conf import settings
try:
    from django.db.models.expressions import Expression
except ImportError:
    # Django < 1.8
    from django.db.models.sql.expressions import SQLEvaluator as Expression
try:
    from django.utils.encoding import smart_unicode
except ImportError:
    # Python 3
    from django.utils.encoding import smart_text as smart_unicode
from django.utils import translation
from django.db.models.signals import class_prepared
from moneyed import Money, Currency
from moneyed.localization import _FORMATTER, format_money
from djmoney import forms
from djmoney.utils import get_currency_field_name
try:
    from django.db.models.expressions import BaseExpression
except ImportError:
    # Django < 1.8
    from django.db.models.expressions import ExpressionNode as BaseExpression

from djmoney.settings import DEFAULT_CURRENCY, CURRENCY_CHOICES

# If django-money-rates is installed we can automatically
# perform operations with different currencies
try:
    from djmoney_rates.utils import convert_money
    AUTO_CONVERT_MONEY = True
except ImportError:
    AUTO_CONVERT_MONEY = False

from decimal import Decimal, ROUND_DOWN
import inspect

try:
    unicode = unicode
except NameError:
    # 'unicode' is undefined, in Python 3
    basestring = (str, bytes)

__all__ = ('MoneyField', 'NotSupportedLookup')

SUPPORTED_LOOKUPS = ('exact', 'isnull', 'lt', 'gt', 'lte', 'gte')


class NotSupportedLookup(Exception):
    def __init__(self, lookup):
        self.lookup = lookup

    def __str__(self):
        return "Lookup '%s' is not supported for MoneyField" % self.lookup


class MoneyPatched(Money):

    # Set to True or False has a higher priority
    # than USE_L10N == True in the django settings file.
    # The variable "self.use_l10n" has three states:
    use_l10n = None

    def __float__(self):
        return float(self.amount)

    def _convert_to_local_currency(self, other):
        """
        Converts other Money instances to the local currency
        """
        if AUTO_CONVERT_MONEY:
            return convert_money(other.amount, other.currency, self.currency)
        else:
            return other

    @classmethod
    def _patch_to_current_class(cls, money):
        """
        Converts object of type MoneyPatched on the object of type Money.
        """
        return cls(money.amount, money.currency)

    def __pos__(self):
        return MoneyPatched._patch_to_current_class(
            super(MoneyPatched, self).__pos__())

    def __neg__(self):
        return MoneyPatched._patch_to_current_class(
            super(MoneyPatched, self).__neg__())

    def __add__(self, other):
        other = self._convert_to_local_currency(other)
        return MoneyPatched._patch_to_current_class(
            super(MoneyPatched, self).__add__(other))

    def __sub__(self, other):
        other = self._convert_to_local_currency(other)
        return MoneyPatched._patch_to_current_class(
            super(MoneyPatched, self).__sub__(other))

    def __mul__(self, other):

        return MoneyPatched._patch_to_current_class(
            super(MoneyPatched, self).__mul__(other))

    def __eq__(self, other):
        if hasattr(other, 'currency'):
            if self.currency == other.currency:
                return self.amount == other.amount
            raise TypeError('Cannot add or subtract two Money ' +
                            'instances with different currencies.')
        return False

    def __truediv__(self, other):

        if isinstance(other, Money):
            return super(MoneyPatched, self).__truediv__(other)
        else:
            return self._patch_to_current_class(
                super(MoneyPatched, self).__truediv__(other))

    def __rmod__(self, other):

        return MoneyPatched._patch_to_current_class(
            super(MoneyPatched, self).__rmod__(other))

    def __get_current_locale(self):
        # get_language can return None starting on django 1.8
        language = translation.get_language() or settings.LANGUAGE_CODE
        locale = translation.to_locale(language)

        if _FORMATTER.get_formatting_definition(locale):
            return locale

        if _FORMATTER.get_formatting_definition('%s_%s' % (locale, locale)):
            return '%s_%s' % (locale, locale)

        return ''

    def __use_l10n(self):
        """
        Return boolean.
        """
        if self.use_l10n is None:
            return settings.USE_L10N
        return self.use_l10n

    def __unicode__(self):

        if self.__use_l10n():
            locale = self.__get_current_locale()
            if locale:
                return format_money(self, locale=locale)

        return format_money(self)

    def __str__(self):

        if self.__use_l10n():
            locale = self.__get_current_locale()
            if locale:
                return format_money(self, locale=locale)

        return format_money(self)

    def __repr__(self):
        # small fix for tests
        return "%s %s" % (self.amount.to_integral_value(ROUND_DOWN),
                          self.currency)


class MoneyFieldProxy(object):
    def __init__(self, field):
        self.field = field
        self.currency_field_name = get_currency_field_name(self.field.name)

    def _money_from_obj(self, obj):
        amount = obj.__dict__[self.field.name]
        currency = obj.__dict__[self.currency_field_name]
        if amount is None:
            return None
        return MoneyPatched(amount=amount, currency=currency)

    def __get__(self, obj, type=None):
        if obj is None:
            raise AttributeError('Can only be accessed via an instance.')
        if isinstance(obj.__dict__[self.field.name], BaseExpression):
            return obj.__dict__[self.field.name]
        if not isinstance(obj.__dict__[self.field.name], Money):
            obj.__dict__[self.field.name] = self._money_from_obj(obj)
        return obj.__dict__[self.field.name]

    def __set__(self, obj, value):
        if isinstance(value, tuple):
            value = Money(amount=value[0], currency=value[1])
        if isinstance(value, Money):
            obj.__dict__[self.field.name] = value.amount
            # we have to determine whether to replace the currency.
            # i.e. if we do the following:
            # .objects.get_or_create(money_currency='EUR')
            # then the currency is already set up, before this code hits
            # __set__ of MoneyField. This is because the currency field
            # has less creation counter than money field.
            obj_curr = obj.__dict__[self.currency_field_name]
            val_curr = str(value.currency)
            def_curr = str(self.field.default_currency)
            if obj_curr != val_curr:
                # in other words, update the currency only if it wasn't
                # changed before.
                if obj_curr == def_curr:
                    setattr(
                        obj, self.currency_field_name,
                        smart_unicode(value.currency))
        elif isinstance(value, BaseExpression):
            # we can only allow this operation if the currency matches.
            # otherwise, one could use F() with different currencies
            # which should not be possible.
            def _check_currency(obj, field, amt):
                currency_field_name = get_currency_field_name(field.name)
                if obj.__dict__[currency_field_name] != str(amt.currency):
                    raise ValueError(
                        'You cannot use F() with different currencies.')
            if VERSION < (1, 8):
                _check_currency(obj, self.field, value.children[1])
                # Django 1.8 removed `children` attribute.
                if isinstance(value.children[1], Money):
                    value.children[1] = value.children[1].amount
            elif VERSION >= (1, 8, 0):
                _check_currency(obj, self.field, value.rhs.value)
                # value.lhs contains F expression, i.e.
                # F(field)
                # rhs contains our value, however we need to extract the amount
                # it is an anology to the above code (pre Django-1.8)
                value.rhs.value = value.rhs.value.amount
            obj.__dict__[self.field.name] = value
        else:
            if value:
                value = str(value)
            obj.__dict__[self.field.name] = self.field.to_python(value)


class CurrencyField(models.CharField):
    description = "A field which stores currency."

    def __init__(self, price_field=None, verbose_name=None, name=None,
                 default=DEFAULT_CURRENCY, **kwargs):
        if isinstance(default, Currency):
            default = default.code
        kwargs['max_length'] = 3
        self.price_field = price_field
        self.frozen_by_south = kwargs.pop('frozen_by_south', False)
        super(CurrencyField, self).__init__(verbose_name, name, default=default,
                                            **kwargs)

    def get_internal_type(self):
        return "CharField"

    def contribute_to_class(self, cls, name):
        if not self.frozen_by_south and not name in [f.name for f in cls._meta.fields]:
            super(CurrencyField, self).contribute_to_class(cls, name)


class MoneyField(models.DecimalField):
    description = "A field which stores both the currency and amount of money."

    def __init__(self, verbose_name=None, name=None,
                 max_digits=None, decimal_places=None,
                 default=None,
                 default_currency=DEFAULT_CURRENCY,
                 currency_choices=CURRENCY_CHOICES, **kwargs):
        nullable = kwargs.get('null', False)
        if default is None and not nullable:
            # Backwards compatible fix for non-nullable fields
            default = 0.0

        if isinstance(default, basestring):
            try:
                # handle scenario where default is formatted like:
                # 'amount currency-code'
                amount, currency = default.split(" ")
            except ValueError:
                # value error would be risen if the default is
                # without the currency part, i.e
                # 'amount'
                amount = default
                currency = default_currency
            default = Money(Decimal(amount), Currency(code=currency))
        elif isinstance(default, (float, Decimal, int)):
            default = Money(default, default_currency)

        if not (nullable and default is None) and not isinstance(default, Money):
            raise Exception(
                "default value must be an instance of Money, is: %s" % str(
                    default))

        # Avoid giving the user hard-to-debug errors if they miss required attributes
        if max_digits is None:
            raise Exception(
                "You have to provide a max_digits attribute to Money fields.")

        if decimal_places is None:
            raise Exception(
                "You have to provide a decimal_places attribute to Money fields.")

        if not default_currency:
            default_currency = default.currency

        self.default_currency = default_currency
        self.currency_choices = currency_choices
        self.frozen_by_south = kwargs.pop('frozen_by_south', False)

        super(MoneyField, self).__init__(verbose_name, name, max_digits,
                                         decimal_places, default=default,
                                         **kwargs)

    def to_python(self, value):
        if isinstance(value, Expression):
            return value
        if isinstance(value, Money):
            value = value.amount
        if isinstance(value, tuple):
            value = value[0]
        return super(MoneyField, self).to_python(value)

    def get_internal_type(self):
        return "DecimalField"

    def contribute_to_class(self, cls, name):

        cls._meta.has_money_field = True

        # Don't run on abstract classes
        # Removed, see https://github.com/jakewins/django-money/issues/42
        # if cls._meta.abstract:
        #    return

        if not self.frozen_by_south:
            c_field_name = get_currency_field_name(name)
            # Do not change default=self.default_currency.code, needed
            # for south compat.
            c_field = CurrencyField(
                max_length=3, price_field=self,
                default=self.default_currency, editable=False,
                choices=self.currency_choices
            )
            c_field.creation_counter = self.creation_counter
            cls.add_to_class(c_field_name, c_field)

        super(MoneyField, self).contribute_to_class(cls, name)

        setattr(cls, self.name, MoneyFieldProxy(self))

    def get_db_prep_save(self, value, connection):
        if isinstance(value, Expression):
            return value
        if isinstance(value, Money):
            value = value.amount
        return super(MoneyField, self).get_db_prep_save(value, connection)

    def get_db_prep_lookup(self, lookup_type, value, connection,
                           prepared=False):
        if not lookup_type in SUPPORTED_LOOKUPS:
            raise NotSupportedLookup(lookup_type)
        value = self.get_db_prep_save(value, connection)
        return super(MoneyField, self).get_db_prep_lookup(lookup_type, value,
                                                          connection, prepared)

    def get_default(self):
        if isinstance(self.default, Money):
            frm = inspect.stack()[1]
            mod = inspect.getmodule(frm[0])
            # We need to return the numerical value if this is called by south
            if mod.__name__ == "south.db.generic":
                return float(self.default.amount)
            return self.default
        else:
            return super(MoneyField, self).get_default()

    def formfield(self, **kwargs):
        defaults = {'form_class': forms.MoneyField}
        defaults.update(kwargs)
        defaults['currency_choices'] = self.currency_choices
        return super(MoneyField, self).formfield(**defaults)

    def get_south_default(self):
        return '%s' % str(self.default)

    def get_south_default_currency(self):
        return '"%s"' % str(self.default_currency.code)

    def value_to_string(self, obj):
        value = self._get_val_from_obj(obj)
        return self.get_prep_value(value)

    # # South support
    def south_field_triple(self):
        "Returns a suitable description of this field for South."
        # Note: This method gets automatically with schemamigration time.
        from south.modelsinspector import introspector
        field_class = self.__class__.__module__ + "." + self.__class__.__name__
        args, kwargs = introspector(self)
        # We need to
        # 1. Delete the default, 'cause it's not automatically supported.
        kwargs.pop('default')
        # 2. add the default currency, because it's not picked up from the inspector automatically.
        kwargs['default_currency'] = "'%s'" % self.default_currency
        return field_class, args, kwargs

    # # Django 1.7 migration support
    def deconstruct(self):
        name, path, args, kwargs = super(MoneyField, self).deconstruct()

        if self.default is not None:
            kwargs['default'] = self.default.amount
        if self.default_currency != DEFAULT_CURRENCY:
            kwargs['default_currency'] = str(self.default_currency)
        if self.currency_choices != CURRENCY_CHOICES:
            kwargs['currency_choices'] = self.currency_choices
        return name, path, args, kwargs


try:
    from south.modelsinspector import add_introspection_rules
    rules = [
        # MoneyField has its own method.
        ((CurrencyField,),
         [],  # No positional args
         {'default': ('default', {'default': DEFAULT_CURRENCY.code}),
          'max_length': ('max_length', {'default': 3})}),
    ]

    # MoneyField implement the serialization in south_field_triple method
    add_introspection_rules(rules, ["^djmoney\.models\.fields\.CurrencyField"])
except ImportError:
    pass


def patch_managers(sender, **kwargs):
    """
    Patches models managers
    """
    from .managers import money_manager

    if hasattr(sender._meta, 'has_money_field'):
        for _id, name, manager in sender._meta.concrete_managers:
            setattr(sender, name, money_manager(manager))


class_prepared.connect(patch_managers)



from __future__ import unicode_literals
from warnings import warn

from django.core import validators
from django.core.exceptions import ValidationError
from django.forms import MultiValueField, DecimalField, ChoiceField
from moneyed.classes import Money

from .widgets import MoneyWidget
from ..settings import CURRENCY_CHOICES


__all__ = ('MoneyField',)


class MoneyField(MultiValueField):

    # Django 1.5 compat:
    if not hasattr(MultiValueField, 'empty_values'):
        empty_values = list(validators.EMPTY_VALUES)

    def __init__(self, currency_widget=None, currency_choices=CURRENCY_CHOICES,
                 choices=CURRENCY_CHOICES, max_value=None, min_value=None,
                 max_digits=None, decimal_places=None, *args, **kwargs):

        # choices does not make sense in this context, it would mean we had
        # to replace two widgets with one widget dynamically... which is a
        # mess. Instead, we let currency_choices be the same as choices and
        # raise a warning.
        if currency_choices != CURRENCY_CHOICES:
            warn('currency_choices will be deprecated in favor of choices', PendingDeprecationWarning)
            choices = currency_choices

        amount_field = DecimalField(max_value, min_value, max_digits, decimal_places, *args, **kwargs)
        currency_field = ChoiceField(choices=choices)

        # TODO: No idea what currency_widget is supposed to do since it doesn't
        # even receive currency choices as input. Somehow it's supposed to be
        # instantiated from outside. Hard to tell.
        if currency_widget:
            self.widget = currency_widget
        else:
            self.widget = MoneyWidget(
                amount_widget=amount_field.widget,
                currency_widget=currency_field.widget
            )

        # The two fields that this widget comprises
        fields = (amount_field, currency_field)
        super(MoneyField, self).__init__(fields, *args, **kwargs)

    def compress(self, data_list):
        if data_list:
            if not self.required and data_list[0] in self.empty_values:
                return None
            else:
                return Money(*data_list[:2])
        return None

    def has_changed(self, initial, data):
        if initial is None:
            initial = ['' for x in range(0, len(data))]
        else:
            if not isinstance(initial, list):
                initial = self.widget.decompress(initial)

        amount_field, currency_field = self.fields
        amount_initial, currency_initial = initial

        # We treat the amount and currency fields slightly
        # differently: if the amount has changed, then we definitely
        # consider the money value to have changed. If the currency
        # has changed, but the amount is *empty* then we do not
        # consider the money value to have changed. This means that it
        # plays nicely with empty formrows in formsets.
        try:
            amount_data = data[0]
        except IndexError:
            amount_data = None

        try:
            amount_initial = amount_field.to_python(amount_initial)
        except ValidationError:
            return True
        if amount_field._has_changed(amount_initial, amount_data):
            return True

        try:
            currency_data = data[1]
        except IndexError:
            currency_data = None

        try:
            currency_initial = currency_field.to_python(currency_initial)
        except ValidationError:
            return True
        # If the currency is valid, has changed and there is some
        # amount data, then the money value has changed.
        if currency_field._has_changed(currency_initial, currency_data) and amount_data:
            return True

        return False

    _has_changed = has_changed



'''
Created on May 7, 2011

@author: jake
'''
from decimal import Decimal
from warnings import warn

import moneyed
from django.test import TestCase
from moneyed import Money

from .testapp.forms import MoneyForm, OptionalMoneyForm, MoneyModelForm, MoneyFormMultipleCurrencies
from .testapp.models import ModelWithVanillaMoneyField


class MoneyFormTestCase(TestCase):
    def testRender(self):
        warn('Rendering depends on localization.', DeprecationWarning)

    def testValidate(self):
        m = Money(Decimal(10), moneyed.SEK)

        form = MoneyForm({"money_0": m.amount, "money_1": m.currency})

        self.assertTrue(form.is_valid())

        result = form.cleaned_data['money']
        self.assertTrue(isinstance(result, Money))

        self.assertEquals(result.amount, Decimal("10"))
        self.assertEquals(result.currency, moneyed.SEK)
        self.assertEquals(result, m)

    def testAmountIsNotANumber(self):
        form = MoneyForm({"money_0": "xyz*|\\", "money_1": moneyed.SEK})
        self.assertFalse(form.is_valid())

    def testAmountExceedsMaxValue(self):
        form = MoneyForm({"money_0": 10000, "money_1": moneyed.SEK})
        self.assertFalse(form.is_valid())

    def testAmountExceedsMinValue(self):
        form = MoneyForm({"money_0": 1, "money_1": moneyed.SEK})
        self.assertFalse(form.is_valid())

    def testNonExistentCurrency(self):
        m = Money(Decimal(10), moneyed.EUR)
        form = MoneyForm({"money_0": m.amount, "money_1": m.currency})
        self.assertFalse(form.is_valid())

    def testChangedData(self):
        # Form displays first currency pre-selected, and we don't
        # want that to count as changed data.
        form = MoneyForm({"money_0": "", "money_1": moneyed.SEK})
        self.assertEquals(form.changed_data, [])

        # But if user types something it, it should be noticed:
        form2 = MoneyForm({"money_0": "1.23", "money_1": moneyed.SEK})
        self.assertEquals(form2.changed_data, ['money'])


class MoneyFormMultipleCurrenciesTestCase(TestCase):

    def testChangeCurrencyNotAmount(self):
        # If the amount is the same, but the currency changes, then we
        # should consider this to be a change.
        initial_money = Money(Decimal(10), moneyed.SEK)
        new_money = Money(Decimal(10), moneyed.EUR)

        initial = {'money': initial_money}
        data = {'money_0': new_money.amount, 'money_1': new_money.currency}

        form = MoneyFormMultipleCurrencies(data, initial=initial)
        self.assertEquals(form.changed_data, ['money'])


class OptionalMoneyFormTestCase(TestCase):

    # The currency widget means that 'money_1' will always be filled
    # in, but 'money_0' could be absent/empty.
    def testMissingAmount(self):
        form = OptionalMoneyForm({"money_1": moneyed.SEK})
        self.assertTrue(form.is_valid())

    def testEmptyAmount(self):
        form = OptionalMoneyForm({"money_0": "", "money_1": moneyed.SEK})
        self.assertTrue(form.is_valid())

    def testAmountIsNotANumber(self):
        # Should still complain for invalid data
        form = OptionalMoneyForm({"money_0": "xyz*|\\", "money_1": moneyed.SEK})
        self.assertFalse(form.is_valid())


class MoneyModelFormTestCase(TestCase):
    def testSave(self):
        m = Money(Decimal("10"), moneyed.SEK)
        form = MoneyModelForm({"money_0": m.amount, "money_1": m.currency})

        self.assertTrue(form.is_valid())
        model = form.save()

        retrieved = ModelWithVanillaMoneyField.objects.get(pk=model.pk)
        self.assertEqual(m, retrieved.money)



'''
Created on May 7, 2011

@author: jake
'''
from decimal import Decimal
from django.test import TestCase
from django.db.models import F, Q
try:
    from unittest import skipIf
except ImportError:
    # For python2.6 compatibility
    from unittest2 import skipIf
from moneyed import Money
from .testapp.models import (ModelWithVanillaMoneyField,
    ModelRelatedToModelWithMoney, ModelWithChoicesMoneyField, BaseModel, InheritedModel, InheritorModel,
    SimpleModel, NullMoneyFieldModel, ModelWithDefaultAsDecimal, ModelWithDefaultAsFloat, ModelWithDefaultAsInt,
    ModelWithDefaultAsString, ModelWithDefaultAsStringWithCurrency, ModelWithDefaultAsMoney, ModelWithTwoMoneyFields,
    ProxyModel, ModelWithNonMoneyField)
from djmoney.models.fields import MoneyPatched, AUTO_CONVERT_MONEY
import moneyed
from mock import patch


class VanillaMoneyFieldTestCase(TestCase):

    def testSaving(self):

        somemoney = Money("100.0")

        model = ModelWithVanillaMoneyField(money=somemoney)
        model.save()

        retrieved = ModelWithVanillaMoneyField.objects.get(pk=model.pk)

        self.assertEquals(somemoney.currency, retrieved.money.currency)
        self.assertEquals(somemoney, retrieved.money)

        # Try setting the value directly
        retrieved.money = Money(1, moneyed.DKK)
        retrieved.save()
        retrieved = ModelWithVanillaMoneyField.objects.get(pk=model.pk)

        self.assertEquals(Money(1, moneyed.DKK), retrieved.money)

        object = BaseModel.objects.create()
        self.assertEquals(Money(0, 'USD'), object.first_field)
        object = BaseModel.objects.create(first_field='111.2')
        self.assertEquals(Money('111.2', 'USD'), object.first_field)
        object = BaseModel.objects.create(first_field=Money('123', 'PLN'))
        self.assertEquals(Money('123', 'PLN'), object.first_field)

        object = ModelWithDefaultAsDecimal.objects.create()
        self.assertEquals(Money('0.01', 'CHF'), object.money)
        object = ModelWithDefaultAsInt.objects.create()
        self.assertEquals(Money('123', 'GHS'), object.money)
        object = ModelWithDefaultAsString.objects.create()
        self.assertEquals(Money('123', 'PLN'), object.money)
        object = ModelWithDefaultAsStringWithCurrency.objects.create()
        self.assertEquals(Money('123', 'USD'), object.money)
        object = ModelWithDefaultAsFloat.objects.create()
        self.assertEquals(Money('12.05', 'PLN'), object.money)
        object = ModelWithDefaultAsMoney.objects.create()
        self.assertEquals(Money('0.01', 'RUB'), object.money)

    def testRounding(self):
        somemoney = Money("100.0623456781123219")

        model = ModelWithVanillaMoneyField(money=somemoney)
        model.save()

        retrieved = ModelWithVanillaMoneyField.objects.get(pk=model.pk)

        self.assertEquals(somemoney.currency, retrieved.money.currency)
        self.assertEquals(Money("100.06"), retrieved.money)

    def testRelativeAddition(self):
        # test relative value adding
        somemoney = Money(100, 'USD')
        mymodel = ModelWithVanillaMoneyField.objects.create(money=somemoney)
        # duplicate money
        mymodel.money = F('money') + somemoney
        mymodel.save()
        mymodel = ModelWithVanillaMoneyField.objects.get(pk=mymodel.pk)
        self.assertEquals(mymodel.money, 2 * somemoney)
        # subtract everything.
        mymodel.money = F('money') - (2 * somemoney)
        mymodel.save()
        mymodel = ModelWithVanillaMoneyField.objects.get(pk=mymodel.pk)
        self.assertEquals(Money(0, 'USD'), mymodel.money)
        # check that one cannot use different currencies with F()
        mymodel = ModelWithVanillaMoneyField.objects.get(pk=mymodel.pk)
        with self.assertRaises(ValueError):
            # this model has USD as a currency, therefore this should fail.
            mymodel.money = F('money') + Money(100, 'EUR')

    def testComparisonLookup(self):
        ModelWithTwoMoneyFields.objects.create(amount1=Money(1, 'USD'), amount2=Money(2, 'USD'))
        ModelWithTwoMoneyFields.objects.create(amount1=Money(2, 'USD'), amount2=Money(0, 'USD'))
        ModelWithTwoMoneyFields.objects.create(amount1=Money(3, 'USD'), amount2=Money(0, 'USD'))
        ModelWithTwoMoneyFields.objects.create(amount1=Money(4, 'USD'), amount2=Money(0, 'GHS'))
        ModelWithTwoMoneyFields.objects.create(amount1=Money(5, 'USD'), amount2=Money(5, 'USD'))

        qs = ModelWithTwoMoneyFields.objects.filter(amount1=F('amount2'))
        self.assertEquals(1, qs.count())

        qs = ModelWithTwoMoneyFields.objects.filter(amount1__gt=F('amount2'))
        # should yield 2 USD, 3 USD, but not 4 GHS (different currency)
        self.assertEquals(2, qs.count())

        qs = ModelWithTwoMoneyFields.objects.filter(Q(amount1=Money(1, 'USD')) | Q(amount2=Money(0, 'USD')))
        self.assertEquals(3, qs.count())

        qs = ModelWithTwoMoneyFields.objects.filter(Q(amount1=Money(1, 'USD')) | Q(amount1=Money(4, 'USD')) | Q(amount2=Money(0, 'GHS')))
        self.assertEquals(2, qs.count())

        qs = ModelWithTwoMoneyFields.objects.filter(Q(amount1=Money(1, 'USD')) | Q(amount1=Money(5, 'USD')) | Q(amount2=Money(0, 'GHS')))
        self.assertEquals(3, qs.count())

        qs = ModelWithTwoMoneyFields.objects.filter(Q(amount1=Money(1, 'USD')) | Q(amount1=Money(4, 'USD'), amount2=Money(0, 'GHS')))
        self.assertEquals(2, qs.count())

        qs = ModelWithTwoMoneyFields.objects.filter(Q(amount1=Money(1, 'USD')) | Q(amount1__gt=Money(4, 'USD'), amount2=Money(0, 'GHS')))
        self.assertEquals(1, qs.count())

        qs = ModelWithTwoMoneyFields.objects.filter(Q(amount1=Money(1, 'USD')) | Q(amount1__gte=Money(4, 'USD'), amount2=Money(0, 'GHS')))
        self.assertEquals(2, qs.count())

    def testExactMatch(self):

        somemoney = Money("100.0")

        model = ModelWithVanillaMoneyField()
        model.money = somemoney

        model.save()

        retrieved = ModelWithVanillaMoneyField.objects.get(money=somemoney)

        self.assertEquals(model.pk, retrieved.pk)

    def testRangeSearch(self):

        minMoney = Money("3")

        model = ModelWithVanillaMoneyField(money=Money("100.0"))

        model.save()

        retrieved = ModelWithVanillaMoneyField.objects.get(money__gt=minMoney)
        self.assertEquals(model.pk, retrieved.pk)

        shouldBeEmpty = ModelWithVanillaMoneyField.objects.filter(money__lt=minMoney)
        self.assertEquals(shouldBeEmpty.count(), 0)

    def testCurrencySearch(self):

        otherMoney = Money("1000", moneyed.USD)
        correctMoney = Money("1000", moneyed.ZWN)

        model = ModelWithVanillaMoneyField(money=Money("100.0", moneyed.ZWN))
        model.save()

        shouldBeEmpty = ModelWithVanillaMoneyField.objects.filter(money__lt=otherMoney)
        self.assertEquals(shouldBeEmpty.count(), 0)

        shouldBeOne = ModelWithVanillaMoneyField.objects.filter(money__lt=correctMoney)
        self.assertEquals(shouldBeOne.count(), 1)

    def testCurrencyChoices(self):

        otherMoney = Money("1000", moneyed.USD)
        correctMoney = Money("1000", moneyed.ZWN)

        model = ModelWithChoicesMoneyField(
            money=Money("100.0", moneyed.ZWN)
        )
        model.save()

        shouldBeEmpty = ModelWithChoicesMoneyField.objects.filter(money__lt=otherMoney)
        self.assertEquals(shouldBeEmpty.count(), 0)

        shouldBeOne = ModelWithChoicesMoneyField.objects.filter(money__lt=correctMoney)
        self.assertEquals(shouldBeOne.count(), 1)

        model = ModelWithChoicesMoneyField(
            money=Money("100.0", moneyed.USD)
        )
        model.save()

    def testIsNullLookup(self):

        null_instance = NullMoneyFieldModel.objects.create(field=None)
        null_instance.save()

        normal_instance = NullMoneyFieldModel.objects.create(field=Money(100, 'USD'))
        normal_instance.save()

        shouldBeOne = NullMoneyFieldModel.objects.filter(field=None)
        self.assertEquals(shouldBeOne.count(), 1)

    def testNullDefault(self):
        null_instance = NullMoneyFieldModel.objects.create()
        self.assertEquals(null_instance.field, None)

    def test_get_or_create_respects_currency(self):
        instance, created = ModelWithVanillaMoneyField.objects.get_or_create(
            money_currency='PLN'
        )

        self.assertEquals(str(instance.money.currency), 'PLN', 'currency should be taken into account in get_or_create')

        instance, created = ModelWithVanillaMoneyField.objects.get_or_create(
            money=Money(0, 'EUR')
        )

        self.assertEquals(str(instance.money.currency), 'EUR', 'currency should be taken into account in get_or_create')


class RelatedModelsTestCase(TestCase):

    def testFindModelsRelatedToMoneyModels(self):

        moneyModel = ModelWithVanillaMoneyField(money=Money("100.0", moneyed.ZWN))
        moneyModel.save()

        relatedModel = ModelRelatedToModelWithMoney(moneyModel=moneyModel)
        relatedModel.save()

        ModelRelatedToModelWithMoney.objects.get(moneyModel__money=Money("100.0", moneyed.ZWN))
        ModelRelatedToModelWithMoney.objects.get(moneyModel__money__lt=Money("1000.0", moneyed.ZWN))


class NonMoneyTestCase(TestCase):

    def testAllowExpressionNodesWithoutMoney(self):
        """ allow querying on expression nodes that are not Money """
        ModelWithNonMoneyField(money=Money(100.0), desc="hundred").save()
        instance = ModelWithNonMoneyField.objects.filter(desc=F("desc")).get()
        self.assertEqual(instance.desc, "hundred")


class InheritedModelTestCase(TestCase):
    """Test inheritence from a concrete model"""

    def testBaseModel(self):
        self.assertEqual(BaseModel.objects.model, BaseModel)

    def testInheritedModel(self):
        self.assertEqual(InheritedModel.objects.model, InheritedModel)
        moneyModel = InheritedModel(
            first_field=Money("100.0", moneyed.ZWN),
            second_field=Money("200.0", moneyed.USD),
        )
        moneyModel.save()
        self.assertEqual(moneyModel.first_field, Money(100.0, moneyed.ZWN))
        self.assertEqual(moneyModel.second_field, Money(200.0, moneyed.USD))


class InheritorModelTestCase(TestCase):
    """Test inheritence from an ABSTRACT model"""

    def testInheritorModel(self):
        self.assertEqual(InheritorModel.objects.model, InheritorModel)
        moneyModel = InheritorModel(
            price1=Money("100.0", moneyed.ZWN),
            price2=Money("200.0", moneyed.USD),
        )
        moneyModel.save()
        self.assertEqual(moneyModel.price1, Money(100.0, moneyed.ZWN))
        self.assertEqual(moneyModel.price2, Money(200.0, moneyed.USD))


class ManagerTest(TestCase):

    def test_manager(self):
        self.assertTrue(hasattr(SimpleModel, 'objects'))

    def test_objects_creation(self):
        SimpleModel.objects.create(money=Money("100.0", 'USD'))
        self.assertEqual(SimpleModel.objects.count(), 1)


class ProxyModelTest(TestCase):

    def test_instances(self):
        ProxyModel.objects.create(money=Money("100.0", 'USD'))
        self.assertIsInstance(ProxyModel.objects.get(pk=1), ProxyModel)

    def test_patching(self):
        ProxyModel.objects.create(money=Money("100.0", 'USD'))
        # This will fail if ProxyModel.objects doesn't have the patched manager:
        self.assertEqual(ProxyModel.objects.filter(money__gt=Money("50.00", 'GBP')).count(),
                         0)


class DifferentCurrencyTestCase(TestCase):
    """Test sum/sub operations between different currencies"""

    @skipIf(AUTO_CONVERT_MONEY is False, "You need to install django-money-rates to run this test")
    def test_sum(self):
        with patch(
            'djmoney.models.fields.convert_money',
            side_effect=lambda amount, cur_from, cur_to: Money((amount * Decimal(0.88)), cur_to)
        ):
            result = MoneyPatched(10, 'EUR') + Money(1, 'USD')
            self.assertEqual(round(result.amount, 2), 10.88)
            self.assertEqual(result.currency, moneyed.EUR)

    @skipIf(AUTO_CONVERT_MONEY is False, "You need to install django-money-rates to run this test")
    def test_sub(self):
        with patch(
            'djmoney.models.fields.convert_money',
            side_effect=lambda amount, cur_from, cur_to: Money((amount * Decimal(0.88)), cur_to)
        ):
            result = MoneyPatched(10, 'EUR') - Money(1, 'USD')
            self.assertEqual(round(result.amount, 2), 9.23)
            self.assertEqual(result.currency, moneyed.EUR)

    def test_eq(self):
        self.assertEqual(MoneyPatched(1, 'EUR'), Money(1, 'EUR'))
        self.assertNotEqual(MoneyPatched(1, 'EUR'), Money(2, 'EUR'))
        with self.assertRaises(TypeError):
            MoneyPatched(10, 'EUR') == Money(10, 'USD')



'''
Created on May 7, 2011

@author: jake
'''

from djmoney.models.fields import MoneyField
from django.db import models

import moneyed
from decimal import Decimal

from django import VERSION
if VERSION >= (1, 7):
    from reversion import revisions as reversion
else:
    import reversion


class ModelWithVanillaMoneyField(models.Model):
    money = MoneyField(max_digits=10, decimal_places=2)

class ModelWithDefaultAsInt(models.Model):
    money = MoneyField(default=123, max_digits=10, decimal_places=2, default_currency='GHS')

class ModelWithDefaultAsStringWithCurrency(models.Model):
    money = MoneyField(default='123 USD', max_digits=10, decimal_places=2)

    class Meta:
        verbose_name = 'model_default_string_currency'

class ModelWithDefaultAsString(models.Model):
    money = MoneyField(default='123', max_digits=10, decimal_places=2, default_currency='PLN')

class ModelWithDefaultAsFloat(models.Model):
    money = MoneyField(default=12.05, max_digits=10, decimal_places=2, default_currency='PLN')

class ModelWithDefaultAsDecimal(models.Model):
    money = MoneyField(default=Decimal('0.01'), max_digits=10, decimal_places=2, default_currency='CHF')

class ModelWithDefaultAsMoney(models.Model):
    money = MoneyField(default=moneyed.Money('0.01', 'RUB'), max_digits=10, decimal_places=2)

class ModelWithTwoMoneyFields(models.Model):
    amount1 = MoneyField(max_digits=10, decimal_places=2)
    amount2 = MoneyField(max_digits=10, decimal_places=3)

class ModelRelatedToModelWithMoney(models.Model):
    moneyModel = models.ForeignKey(ModelWithVanillaMoneyField)


class ModelWithChoicesMoneyField(models.Model):
    money = MoneyField(
        max_digits=10,
        decimal_places=2,
        currency_choices=[
            (moneyed.USD, 'US Dollars'),
            (moneyed.ZWN, 'Zimbabwian')
        ],
    )


class ModelWithNonMoneyField(models.Model):
    money = MoneyField(max_digits=10, decimal_places=2, default_currency='USD')
    desc = models.CharField(max_length=10)


class AbstractModel(models.Model):
    price1 = MoneyField(max_digits=10, decimal_places=2, default_currency='USD')

    class Meta:
        abstract = True


class InheritorModel(AbstractModel):
    price2 = MoneyField(max_digits=10, decimal_places=2, default_currency='USD')


class RevisionedModel(models.Model):
    amount = MoneyField(max_digits=10, decimal_places=2, default_currency='USD')

reversion.register(RevisionedModel)


class BaseModel(models.Model):
    first_field = MoneyField(max_digits=10, decimal_places=2, default_currency='USD')


class InheritedModel(BaseModel):
    second_field = MoneyField(max_digits=10, decimal_places=2, default_currency='USD')


class SimpleModel(models.Model):
    money = MoneyField(max_digits=10, decimal_places=2, default_currency='USD')


class NullMoneyFieldModel(models.Model):
    field = MoneyField(max_digits=10, decimal_places=2, null=True)


class ProxyModel(SimpleModel):
    class Meta:
        proxy = True



from django import template
from django.template import TemplateSyntaxError
from moneyed import Money
from decimal import Decimal

from ..models.fields import MoneyPatched

register = template.Library()


class MoneyLocalizeNode(template.Node):

    def __repr__(self):
        return "<MoneyLocalizeNode %r>" % self.money

    def __init__(self, money=None, amount=None, currency=None, use_l10n=None,
                 var_name=None):

        if money and (amount or currency):
            raise Exception('You can define either "money" or the'
                            ' "amount" and "currency".')

        self.money = money
        self.amount = amount
        self.currency = currency
        self.use_l10n = use_l10n
        self.var_name = var_name

    @classmethod
    def handle_token(cls, parser, token):

        tokens = token.contents.split()

        # default value
        var_name = None
        use_l10n = True

        # GET variable var_name
        if len(tokens) > 3:
            if tokens[-2] == 'as':
                var_name = parser.compile_filter(tokens[-1])
                # remove the already used data
                tokens = tokens[0:-2]

        # GET variable use_l10n
        if tokens[-1].lower() in ('on', 'off'):

            if tokens[-1].lower() == 'on':
                use_l10n = True
            else:
                use_l10n = False
            # remove the already used data
            tokens.pop(-1)

        # GET variable money
        if len(tokens) == 2:
            return cls(money=parser.compile_filter(tokens[1]),
                       var_name=var_name, use_l10n=use_l10n)

        # GET variable amount and currency
        if len(tokens) == 3:
            return cls(amount=parser.compile_filter(tokens[1]),
                       currency=parser.compile_filter(tokens[2]),
                       var_name=var_name, use_l10n=use_l10n)

        raise TemplateSyntaxError('Wrong number of input data to the tag.')

    def render(self, context):

        money = self.money.resolve(context) if self.money else None
        amount = self.amount.resolve(context) if self.amount else None
        currency = self.currency.resolve(context) if self.currency else None

        if money is not None:
            if isinstance(money, Money):
                money = MoneyPatched._patch_to_current_class(money)
            else:
                raise TemplateSyntaxError('The variable "money" must be an '
                                          'instance of Money.')

        elif amount is not None and currency is not None:
            money = MoneyPatched(Decimal(str(amount)), str(currency))
        else:
            raise TemplateSyntaxError('You must define both variables: '
                                      'amount and currency.')

        money.use_l10n = self.use_l10n

        if self.var_name is None:
            return money

        # as <var_name>
        context[self.var_name.token] = money
        return ''


@register.tag
def money_localize(parser, token):
    """
    Usage::

        {% money_localize <money_object> [ on(default) | off ] [as var_name] %}
        {% money_localize <amount> <currency> [ on(default) | off ] [as var_name] %}

    Example:

        The same effect:
        {% money_localize money_object %}
        {% money_localize money_object on %}

        Assignment to a variable:
        {% money_localize money_object on as NEW_MONEY_OBJECT %}

        Formatting the number with currency:
        {% money_localize '4.5' 'USD' %}

    Return::

        MoneyPatched object

    """
    return MoneyLocalizeNode.handle_token(parser, token)



from django.test import TestCase
from djmoney.tests.testapp.models import RevisionedModel
from moneyed import Money
from django import VERSION
if VERSION >= (1, 7):
    from reversion import revisions as reversion
else:
    import reversion


class ReversionTestCase(TestCase):
    def test_that_can_safely_restore_deleted_object(self):
        model = None
        amount = Money(100, 'GHS')
        with reversion.create_revision():
            model = RevisionedModel.objects.create(amount=amount)
            model.save()
        model.delete()
        version = reversion.get_deleted(RevisionedModel)[0]
        version.revision.revert()
        model = RevisionedModel.objects.get(pk=1)
        self.assertEquals(model.amount, amount)



# -*- encoding: utf-8 -*-
from __future__ import unicode_literals
import warnings

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

warnings.simplefilter('ignore', Warning)

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'djmoney',
    'testapp'
)

SITE_ID = 1
ROOT_URLCONF = 'core.urls'

SECRET_KEY = 'foobar'

USE_L10N = True

import moneyed
from moneyed.localization import _FORMATTER, DEFAULT
from decimal import ROUND_HALF_EVEN

_FORMATTER.add_sign_definition('pl_PL', moneyed.PLN, suffix=' zł')
_FORMATTER.add_sign_definition(DEFAULT, moneyed.PLN, suffix=' zł')
_FORMATTER.add_formatting_definition(
    "pl_PL", group_size=3, group_separator=" ", decimal_point=",",
    positive_sign="", trailing_positive_sign="",
    negative_sign="-", trailing_negative_sign="",
    rounding_method=ROUND_HALF_EVEN)



#-*- encoding: utf-8 -*-
from setuptools import setup

from setuptools.command.test import test as TestCommand
import sys

class Tox(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True
    def run_tests(self):
        #import here, cause outside the eggs aren't loaded
        import tox
        errno = tox.cmdline(self.test_args)
        sys.exit(errno)


setup(name="django-money",
      version="0.7.6",
      description="Adds support for using money and currency fields in django models and forms. Uses py-moneyed as the money implementation.",
      url="https://github.com/jakewins/django-money",
      maintainer='Greg Reinbach',
      maintainer_email='greg@reinbach.com',
      packages=["djmoney",
                "djmoney.forms",
                "djmoney.models",
                "djmoney.templatetags",
                "djmoney.tests"],
      install_requires=['setuptools',
                        'Django >= 1.4',
                        'py-moneyed > 0.4',
                        'six'],
      platforms=['Any'],
      keywords=['django', 'py-money', 'money'],
      classifiers=["Development Status :: 5 - Production/Stable",
                   "Intended Audience :: Developers",
                   "License :: OSI Approved :: BSD License",
                   "Operating System :: OS Independent",
                   "Programming Language :: Python",
                   "Programming Language :: Python :: 3",
                   "Framework :: Django", ],
      tests_require=['tox>=1.6.0'],
      cmdclass={'test': Tox},
      )



from django.test import TestCase
from django.db.models import Q
from djmoney.models.managers import _expand_money_args
from djmoney.tests.testapp.models import ModelWithNonMoneyField
from moneyed import Money


class ExpandMoneyArgsTestCase(TestCase):

    def testExpandMoneyArgs(self):
        # Test no args
        self.assertEqual(
            _expand_money_args(ModelWithNonMoneyField(), []),
            []
        )

        # Test non-Q arg (such as it receives for an order_by
        self.assertEqual(
            _expand_money_args(ModelWithNonMoneyField(), ['money']),
            ['money']
        )

        # Test
        #   (AND: ('money', 0 USD))
        # results in;
        #   (AND: (AND: ('money', 0 USD), ('money_currency', u'USD')))
        actual = _expand_money_args(ModelWithNonMoneyField(), [Q(money=Money(0, 'USD'))])

        self.assertEqual(len(actual), 1)
        arg = actual[0]

        self.assertIsInstance(arg, Q)
        self.assertEqual(arg.connector, Q.AND)
        self.assertEqual(len(arg.children), 1)
        self.assertIsInstance(arg.children[0], Q)
        self.assertEqual(arg.children[0].connector, Q.AND)
        self.assertIn(('money', Money(0, 'USD')), arg.children[0].children)
        self.assertIn(('money_currency', 'USD'), arg.children[0].children)

        # Test
        #   (AND: ('desc', 'foo'), ('money', 0 USD))
        # results in;
        #   (AND: ('desc', 'foo'), (AND: ('money', 0 USD), ('money_currency', u'USD')))
        actual = _expand_money_args(ModelWithNonMoneyField(), [Q(money=Money(0, 'USD'), desc='foo')])
        self.assertEqual(len(actual), 1)

        arg = actual[0]

        self.assertIsInstance(arg, Q)
        self.assertEqual(arg.connector, Q.AND)
        self.assertEqual(len(arg.children), 2)

        # Can't guarantee the ordering of children, thus;
        for child in arg.children:
            if isinstance(child, tuple):
                self.assertEqual(('desc', 'foo'), child)
            elif isinstance(child, Q):
                self.assertEqual(child.connector, Q.AND)
                self.assertEqual(len(child.children), 2)
                self.assertIn(('money', Money(0, 'USD')), child.children)
                self.assertIn(('money_currency', 'USD'), child.children)
            else:
                self.fail("There should only be two elements, a tuple and a Q - not a %s" % child)

        # Test
        #   (OR: (AND: ('desc', 'foo'), ('money', 0 USD)), ('desc', 'bar'))
        # results in:
        #   (OR: (AND: ('desc', 'foo'), (AND: ('money', 0 USD), ('money_currency', u'USD'))), ('desc', 'bar'))
        actual = _expand_money_args(ModelWithNonMoneyField(), [Q(money=Money(0, 'USD'), desc='foo') | Q(desc='bar')])

        self.assertEqual(len(actual), 1)
        arg = actual[0]

        self.assertIsInstance(arg, Q)
        self.assertEqual(arg.connector, Q.OR)
        self.assertEqual(len(arg.children), 2)

        # Can't guarantee the ordering of children, thus;
        for child in arg.children:
            if isinstance(child, tuple):
                self.assertEqual(('desc', 'bar'), child)
            elif isinstance(child, Q):
                self.assertEqual(len(child.children), 2)
                for subchild in child.children:
                    if isinstance(subchild, tuple):
                        self.assertEqual(('desc', 'foo'), subchild)
                    elif isinstance(subchild, Q):
                        self.assertIn(('money', Money(0, 'USD')), subchild.children)
                        self.assertIn(('money_currency', 'USD'), subchild.children)
            else:
                self.fail("There should only be two elements, a tuple and a Q - not a %s" % child)

        # Test
        #   (OR: (OR: (AND: ('desc', 'foo'), ('money', 0 USD)), ('desc', 'eggs')), ('desc', 'bar'))
        # results in;
        #   (OR: (OR: (AND: ('desc', 'foo'), (AND: ('money', 0 USD), ('money_currency', u'USD'))), ('desc', 'eggs')), ('desc', 'bar'))
        actual = _expand_money_args(ModelWithNonMoneyField(), [Q(Q(money=Money(0, 'USD'), desc='foo') | Q(desc='eggs')) | Q(desc='bar')])
        arg = actual[0]

        self.assertEqual(len(actual), 1)
        self.assertIsInstance(arg, Q)
        self.assertEqual(arg.connector, Q.OR)
        self.assertEqual(len(arg.children), 2)

        # Can't guarantee the ordering of children, thus;
        for child in arg.children:
            if isinstance(child, tuple):
                self.assertEqual(('desc', 'bar'), child)
            elif isinstance(child, Q):
                self.assertEqual(len(child.children), 2)
                for subchild in child.children:
                    if isinstance(subchild, tuple):
                        self.assertEqual(('desc', 'eggs'), subchild)
                    elif isinstance(subchild, Q):
                        for subsubchild in subchild.children:
                            if isinstance(subsubchild, tuple):
                                self.assertEqual(('desc', 'foo'), subsubchild)
                            elif isinstance(subsubchild, Q):
                                self.assertIn(('money', Money(0, 'USD')), subsubchild.children)
                                self.assertIn(('money_currency', 'USD'), subsubchild.children)
                            else:
                                self.fail("There should only be two subsubchild elements, a tuple and a Q - not a %s" % subsubchild)
                    else:
                        self.fail("There should only be two subchild elements, a tuple and a Q - not a %s" % subsubchild)
            else:
                self.fail("There should only be two child elements, a tuple and a Q - not a %s" % child)



from django.contrib import admin

import models

class InheritedModelAdmin(admin.ModelAdmin):
     readonly_fields = ('second_field',)

admin.site.register(models.InheritedModel, InheritedModelAdmin)



# -*- encoding: utf-8 -*-
from __future__ import unicode_literals
from django.test import TestCase
from django import template
from django.utils import translation

from ..models.fields import MoneyPatched
from moneyed import Money


class MoneyLocalizeTestCase(TestCase):

    def setUp(self):
        self.default_language = translation.get_language()
        translation.activate('pl')
        super(TestCase, self).setUp()

    def tearDown(self):
        translation.activate(self.default_language)
        super(TestCase, self).tearDown()

    def assertTemplate(self, template_string, result, context={}):
        c = template.Context(context)
        t = template.Template(template_string)
        self.assertEqual(t.render(c), result)

    def testOnOff(self):

        # with a tag template "money_localize"
        self.assertTemplate(
            '{% load djmoney %}{% money_localize money %}',
            '2,30 zł',
            context={'money': Money(2.3, 'PLN')})

        # without a tag template "money_localize"
        self.assertTemplate(
            '{{ money }}',
            '2,30 zł',
            context={'money': MoneyPatched(2.3, 'PLN')})

        with self.settings(USE_L10N=False):
            # money_localize has a default setting USE_L10N = True
            self.assertTemplate(
                '{% load djmoney %}{% money_localize money %}',
                '2,30 zł',
                context={'money': Money(2.3, 'PLN')})

            # without a tag template "money_localize"
            self.assertTemplate(
                '{{ money }}',
                '2.30 zł',
                context={'money': MoneyPatched(2.3, 'PLN')})
            mp = MoneyPatched(2.3, 'PLN')
            mp.use_l10n = True
            self.assertTemplate(
                '{{ money }}',
                '2,30 zł',
                context={'money': mp})

        self.assertTemplate(
            '{% load djmoney %}{% money_localize money on %}',
            '2,30 zł',
            context={'money': Money(2.3, 'PLN')})

        with self.settings(USE_L10N=False):
            self.assertTemplate(
                '{% load djmoney %}{% money_localize money on %}',
                '2,30 zł',
                context={'money': Money(2.3, 'PLN')})

        self.assertTemplate(
            '{% load djmoney %}{% money_localize money off %}',
            '2.30 zł',
            context={'money': Money(2.3, 'PLN')})

    def testAsVar(self):

        self.assertTemplate(
            '{% load djmoney %}{% money_localize money as NEW_M %}{{NEW_M}}',
            '2,30 zł',
            context={'money': Money(2.3, 'PLN')})

        self.assertTemplate(
            '{% load djmoney %}{% money_localize money off as NEW_M %}{{NEW_M}}',
            '2.30 zł',
            context={'money': Money(2.3, 'PLN')})

        # test zero amount of money
        self.assertTemplate(
            '{% load djmoney %}{% money_localize money off as NEW_M %}{{NEW_M}}',
            '0.00 zł',
            context={'money': Money(0, 'PLN')})

    def testConvert(self):

        self.assertTemplate(
            '{% load djmoney %}{% money_localize "2.5" "PLN" as NEW_M %}{{NEW_M}}',
            '2,50 zł',
            context={})

        self.assertTemplate(
            '{% load djmoney %}{% money_localize "2.5" "PLN" %}',
            '2,50 zł',
            context={})

        self.assertTemplate(
            '{% load djmoney %}{% money_localize amount currency %}',
            '2,60 zł',
            context={'amount': 2.6, 'currency': 'PLN'})



'''
Created on May 15, 2011

@author: jake
'''

from django import forms
from djmoney.forms import MoneyField
from .models import ModelWithVanillaMoneyField


class MoneyForm(forms.Form):

    money = MoneyField(currency_choices=[(u'SEK', u'Swedish Krona')], max_value=1000, min_value=2)

class MoneyFormMultipleCurrencies(forms.Form):

    money = MoneyField(currency_choices=[(u'SEK', u'Swedish Krona'), (u'EUR', u'Euro')], max_value=1000, min_value=2)

class OptionalMoneyForm(forms.Form):

    money = MoneyField(required=False, currency_choices=[(u'SEK', u'Swedish Krona')])

class MoneyModelForm(forms.ModelForm):

    class Meta:
        model = ModelWithVanillaMoneyField
        fields = ('money',)
